import logging

from rest_framework import generics

from core.models.rooms import Room

from api.serializers import RoomSerializer
from api.views.mixins import ConferenceSlugMixin

logger = logging.getLogger(__name__)


class ConferenceRoomList(ConferenceSlugMixin, generics.ListAPIView):
    serializer_class = RoomSerializer

    def get_queryset(self, **kwargs):
        return Room.objects.conference_accessible(conference=self.conference).order_by('name')


class ConferenceRoomDetail(ConferenceSlugMixin, generics.RetrieveAPIView):
    serializer_class = RoomSerializer

    def get_object(self, **kwargs):
        room_id = self.request.resolver_match.kwargs['pk']
        return Room.objects.conference_accessible(conference=self.conference).get(pk=room_id)
