import uuid
from datetime import UTC, datetime

from django.contrib.auth.models import Group
from django.test import TestCase, override_settings

from core.models import Conference, ConferenceMember, PlatformUser

TEST_CONF_ID = uuid.uuid4()


@override_settings(SELECTED_CONFERENCE_ID=TEST_CONF_ID)
class BackOfficeTestCase(TestCase):
    fixtures = [
        'bootstrap_auth_groups',
    ]

    def setUp(self):
        self.conf = Conference(
            id=TEST_CONF_ID,
            name='conf_asdf',
            slug='slug1',
            start=datetime(2020, 1, 1, 0, 0, 0, tzinfo=UTC),
            end=datetime(2020, 1, 3, 0, 0, 0, tzinfo=UTC),
            is_public=True,
        )
        self.conf.save()
        self.user = PlatformUser(username='testuser', email='no@where.test', is_staff=True, is_superuser=True)
        self.user.save()
        self.conference_member = ConferenceMember(conference=self.conf, user=self.user)
        self.conference_member.save()
        self.staff = PlatformUser.objects.create(username='test_staff', email='staff@where.test')
        self.staff_cm = ConferenceMember.objects.create(conference=self.conf, user=self.staff, is_staff=True)
        self.staff_cm.permission_groups.add(Group.objects.get(name='Assembly-Team'))
        self.non_staff_user = PlatformUser.objects.create(username='test_non_staff', email='no@where.test')
        self.non_staff_cm = ConferenceMember.objects.create(conference=self.conf, user=self.non_staff_user)

    def tearDown(self) -> None:
        self.client.logout()
