from typing import Any

from django.http import Http404, HttpRequest
from django.http.response import HttpResponse
from django.views.generic import CreateView, ListView, UpdateView

from core.models import Invitation

from backoffice.views.invitations.habitat import InvitationHabitatSendView, InvitationHabitatUpdateView


class InvitationListView(ListView):
    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        raise Http404('View not yet implemented')


class InvitationUpdateView(UpdateView):
    model = Invitation

    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        invitation = self.get_object()
        if not Invitation.type_is(invitation):  # pragma: no cover
            raise ValueError('Invalid invitation type')
        match invitation.type:
            case Invitation.InvitationType.HABITAT:
                return InvitationHabitatUpdateView.as_view()(request, *args, **kwargs)
            case _:  # pragma: no cover
                raise Http404('View not yet implemented')


class InvitationSendView(CreateView):
    model = Invitation

    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        match kwargs.get('type'):
            case 'habitat':
                return InvitationHabitatSendView.as_view()(request, *args, **kwargs)
            case 'assembly_member':
                raise Http404('View not yet implemented')
            case 'team_member':
                raise Http404('View not yet implemented')
            case _:
                return HttpResponse('Invalid invitation type', status=400)


__all__ = [
    'InvitationListView',
    'InvitationSendView',
    'InvitationUpdateView',
]
