import logging

from django.contrib import messages
from django.shortcuts import redirect
from django.views.generic import DetailView, ListView

from core.models import Badge

from backoffice.views.moderation.mixins import ModerationAdminMixin

logger = logging.getLogger(__name__)


class ModerationBadgeListView(ModerationAdminMixin, ListView):
    template_name = 'backoffice/moderation_badge-list.html'

    def get_queryset(self):
        qs = self.conference.badges.select_related('category', 'issuing_assembly').order_by('name').all()
        return qs


class ModerationBadgeDetailView(ModerationAdminMixin, DetailView):
    template_name = 'backoffice/moderation_badge-detail.html'

    def get_queryset(self):
        qs = self.conference.badges.all()
        return qs

    def post(self, request, *args, **kwargs):
        badge = self.get_object()  # type: Badge
        action = request.POST.get('action')

        # TODO: add log entries
        if action == 'hide':
            badge.state = Badge.State.HIDDEN
            badge.save(update_fields=['state'])
            messages.success(request, f'badge #{badge.id} state set to "hidden"')

        elif action == 'delete':
            badge_id = badge.id
            badge.delete()
            messages.success(request, f'badge #{badge_id} deleted: {badge.name}')
            return redirect('backoffice:moderation-badge-list')

        else:
            messages.error(request, f'UNKNOWN ACTION: {action}')

        return redirect('backoffice:moderation-badge-detail', pk=badge.pk)
