import logging

from django.contrib import messages
from django.shortcuts import redirect
from django.views.generic import DetailView, ListView

from backoffice.views.moderation.mixins import ModerationAdminMixin

logger = logging.getLogger(__name__)


class ModerationEventListView(ModerationAdminMixin, ListView):
    template_name = 'backoffice/event/list.html'
    context_object_name = 'events'

    def get_queryset(self):
        qs = self.conference.events.select_related('assembly').order_by('schedule_start', 'name').all()
        return qs


class ModerationEventDetailView(ModerationAdminMixin, DetailView):
    template_name = 'backoffice/moderation_event-detail.html'

    def get_queryset(self):
        qs = self.conference.events.all()
        return qs

    def post(self, request, *args, **kwargs):
        event = self.get_object()
        action = request.POST.get('action')

        # TODO: add log entries
        if action == 'hide':
            event.is_public = False
            event.save(update_fields=['is_public'])
            messages.success(request, f'event set non-public: {event.id}')

        elif action == 'delete':
            event_id = event.id
            event.delete()
            messages.success(request, f'event entry deleted: {event_id}')
            return redirect('backoffice:moderation-event-list')

        else:
            messages.error(request, f'UNKNOWN ACTION: {action}')

        return redirect('backoffice:moderation-event-detail', pk=event.pk)
