from django.db import models
from django.utils.translation import gettext_lazy as _


class ConferenceReference(models.ForeignKey):
    def __init__(self, **kwargs):
        options = {
            'on_delete': models.CASCADE,
            'help_text': _('Conference__reference_help'),
            'verbose_name': _('Conference__reference'),
            'to': 'core.Conference',
        }
        if kwargs is not None:
            options.update(kwargs)
        super().__init__(**options)


class FskField(models.CharField):
    class FSK(models.TextChoices):
        FSK_0 = 'all', _('FSK_0')
        FSK_14 = '14+', _('FSK_14')
        FSK_16 = '16+', _('FSK_16')
        FSK_18 = '18+', _('FSK_18')

    def __init__(self, **kwargs):
        options = {
            'max_length': 10,
            'choices': self.FSK.choices,
            'default': self.FSK.FSK_18,
            'help_text': _('FSK__help'),
            'verbose_name': _('FSK'),
        }
        if kwargs is not None:
            options.update(kwargs)
        super().__init__(**options)
