from django.core.management.base import BaseCommand

from core.models.assemblies import AssemblyMember
from core.models.conference import Conference
from core.models.users import UserCommunicationChannel


class Command(BaseCommand):
    help = "List all assemblies' contacts' verified e-mail addresses"

    def add_arguments(self, parser):
        parser.add_argument('conf_slug', help='conference slug')

    def handle(self, *args, **options):
        conf = Conference.objects.get(slug=options.get('conf_slug'))

        addresses = set()
        members = AssemblyMember.objects.filter(assembly__conference=conf)
        for m in members:
            u = m.member
            if not u.is_active:
                continue
            addresses.add(u.email)
            for c in u.communication_channels.filter(is_verified=True, channel=UserCommunicationChannel.Channel.MAIL):
                addresses.add(c.address)

        if '' in addresses:
            addresses.remove('')

        for a in sorted(addresses):
            print(a)
