# Generated by Django 3.1.3 on 2020-11-28 07:37

from django.db import migrations


def set_event_venues(apps, schema_editor):
    Event = apps.get_model('core', 'Event')
    Venue = apps.get_model('core', 'Venue')

    # attempt to set event's venue based on event's room (which belongs to an assembly)
    candidates = set()
    for event in Event.objects.all():
        if event.room is not None:
            event.venue = event.room.venue
            event.save()
            continue

        candidates.add(event.conference)

    # in each conference create a placeholder assembly and move all remaining events without an associated venue there
    for conf in candidates:
        venue = Venue(conference=conf, slug='MIGRATION', name='TODO DB MIGRATION', description='Placeholder for all events without an associated assembly.')
        venue.save()
        Event.objects.filter(conference=conf, venue_id=None).update(venue_id=venue.id)

    for event in Event.objects.filter(venue=None).all():
        venue, venue_created = Venue.objects.get_or_create(
            conference=event.conference,
            slug='MIGRATION',
            name='TODO DB MIGRATION',
            description='Placeholder for all events without an associated assembly.',
        )
        if venue_created:
            venue.save()

        event.venue = venue
        event.save()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0017_schema_extensions'),
    ]

    operations = [
        migrations.RunPython(set_event_venues, reverse_code=migrations.RunPython.noop),
    ]
