from django.contrib import admin
from modeltranslation.admin import TranslationAdmin
from modeltranslation.translator import TranslationOptions, register

from core.admin import BadgeAdmin, BadgeCategoryAdmin, ProjectAdmin
from core.models import (
    Assembly,
    Badge,
    BadgeCategory,
    BulletinBoardEntry,
    ConferenceMember,
    ConferenceNavigationItem,
    Event,
    MapFloor,
    MapPOI,
    MetaNavItem,
    Project,
    Room,
)


@register(Assembly)
class AssemblyTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
    )


@register(Badge)
class BadgeTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
        'location',
    )


class TranslatedBadgeAdmin(BadgeAdmin, TranslationAdmin):
    pass


admin.site.unregister(Badge)
admin.site.register(Badge, TranslatedBadgeAdmin)


@register(BadgeCategory)
class BadgeCategoryTranslationOptions(TranslationOptions):
    fields = (
        'name',
        'description',
    )


class TranslatedBadgeCategoryAdmin(BadgeCategoryAdmin, TranslationAdmin):
    pass


admin.site.unregister(BadgeCategory)
admin.site.register(BadgeCategory, TranslatedBadgeCategoryAdmin)


@register(BulletinBoardEntry)
class BulletinBoardEntryTranslationOptions(TranslationOptions):
    fields = (
        'text',
        'text_html',
    )


@register(ConferenceNavigationItem)
class ConferenceNavigationItemTranslationOptions(TranslationOptions):
    fields = (
        'label',
        'title',
    )


@register(Event)
class EventTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
    )


@register(ConferenceMember)
class ConferenceMemberTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
    )


@register(Room)
class RoomTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
    )


@register(MapFloor)
class MapFloorTranslationOptions(TranslationOptions):
    fields = ('name',)


@register(MapPOI)
class MapPOITranslationOptions(TranslationOptions):
    fields = (
        'name',
        'description',
        'description_html',
    )


@register(MetaNavItem)
class MetaNavItemTranslationOptions(TranslationOptions):
    fields = ('title',)


@register(Project)
class ProjectTranslationOptions(TranslationOptions):
    fields = (
        'description',
        'description_html',
    )


class TranslatedProjectAdmin(ProjectAdmin, TranslationAdmin):
    pass


admin.site.unregister(Project)
admin.site.register(Project, TranslatedProjectAdmin)
