import importlib.util
import os
import sys

import environ

dev_env = environ.FileAwareEnv(
    SERVE_DEBUGPY=(bool, False),
    FORCE_DEBUG_TOOLBAR=(bool, False),
)

if dev_env('SERVE_DEBUGPY'):
    # Disable debug warnings for python 3.11 see https://github.com/microsoft/debugpy/issues/861
    os.environ.setdefault('PYDEVD_DISABLE_FILE_VALIDATION', '1')

    try:
        import debugpy  # noqa: T100

        debugpy.listen(('0.0.0.0', 5678))  # noqa: T100
        print('debugpy ready for connection at 0.0.0.0:5678')

    except ImportError:
        print('!' * 72)
        print('! SERVE_DEBUGPY is enabled, but `debugpy` is not installed ')
        print('!' * 72)
    except RuntimeError:
        pass
# activate SECRET_KEY-from-file mode by defaulting to DEBUG mode
os.environ.setdefault('DJANGO_DEBUG', 'I_KNOW_WHAT_I_AM_DOING')

# set default ALLOWED_HOSTS
os.environ.setdefault('ALLOWED_HOSTS', 'localhost')

# set PLAINUI link options
os.environ.setdefault('PLAINUI_BASE_URL', 'http://127.0.0.1:8000')

# activate all parts by default
os.environ.setdefault('SERVE_ADMIN', 'yes')
os.environ.setdefault('SERVE_API', 'yes')
os.environ.setdefault('SERVE_BACKOFFICE', 'yes')
os.environ.setdefault('SERVE_FRONTEND', 'yes')

# serve metrics locally only (and silence the warning)
os.environ.setdefault('METRICS_SERVER_IPS', '127.0.0.1')

# store media files locally
os.environ.setdefault('STORAGE_TYPE', 'local')

# don't use manifest storage
os.environ.setdefault('USE_PLAIN_STATICFILES', 'yes')

# setup info logging
os.environ.setdefault('LOG_LEVEL', 'TRACE')

# setup abuse contacts
os.environ.setdefault('ABUSE_DEFAULT_RECIPIENT', 'report@hub.example.net')
os.environ.setdefault('ABUSE_RECIPIENTS', 'abuse=abuse@hub.example.net,tech=hub@hub.example.net,foo=')

# load all default settings
from .default import *  # noqa: F401, E402, F403 # pylint: disable=wildcard-import,unused-wildcard-import

# force DEBUG mode
DEBUG = True

# "send" emails by printing them on the console
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# configure debug_toolbar if it is installed and we're not testing
if 'test' not in sys.argv:
    if importlib.util.find_spec('debug_toolbar') is not None:
        INSTALLED_APPS += [  # noqa: F405
            'debug_toolbar',
        ]

        MIDDLEWARE += [  # noqa: F405
            'debug_toolbar.middleware.DebugToolbarMiddleware',
        ]

        DEBUG_TOOLBAR_CONFIG = {
            'JQUERY_URL': '',
            'SHOW_TOOLBAR_CALLBACK': lambda _request: True,
        }
        if dev_env('FORCE_DEBUG_TOOLBAR'):
            DEBUG_TOOLBAR_CONFIG['SHOW_TOOLBAR_CALLBACK'] = lambda _request: True

        INTERNAL_IPS = [
            '127.0.0.1',
        ]
    if importlib.util.find_spec('django_watchfiles') is not None:
        INSTALLED_APPS += [  # noqa: F405
            'django_watchfiles',
        ]

# configure extensions if they're available
# allows to e.g. generate a PNG of the models' relationships:
#   ./manage.py graph_models core -g -o ../models.png --disable-abstract-fields -X ContentType,PermissionsMixin,AbstractBaseUser,Group,Permission
if importlib.util.find_spec('django_extensions') is not None:
    INSTALLED_APPS = ['django_extensions', *INSTALLED_APPS]  # noqa: F405
