var initParallax = (callback) => {
  if (document.readyState != "loading") callback();
  else document.addEventListener("DOMContentLoaded", callback);
};

initParallax(() => {
  if (!("querySelector" in document) || !("requestAnimationFrame" in window)) {
    return;
  }
  const body = document.querySelector("body");
  // Parallax only with some body classes
  if (!body.classList.contains("eyecandy-deluxe")) {
    return;
  }

  let lastPos;
  const layer01 = document.querySelector(".rc3-bg-L01");
  const layer05L = document.querySelector(".rc3-bg-L05-L");
  const layer05R = document.querySelector(".rc3-bg-L05-R");
  const layer06 = document.querySelector(".rc3-bg-L06");
  const layer10 = document.querySelector(".rc3-bg-L10");
  const main = document.querySelector(".rc3-main");

  if (layer05R) {
    layer05R.classList.add(`collage-r-` + Math.floor(Math.random() * 7 + 1));
  }

  if (layer05L) {
    layer05L.classList.add(`collage-l-` + Math.floor(Math.random() * 6 + 1));
  }

  const setTransform = (el, factor, scale) => {
    if (el) {
      el.style.transform = `translate3D(0,${window.scrollY * factor}px,0) scale(${scale})`;
    }
  };
  const setBgPosition = (el, factor) => {
    if (el) {
      el.style.backgroundPositionY = window.scrollY * factor + "px";
    }
  };

  function step() {
    if (!lastPos) lastPos = window.scrollY;

    if (lastPos !== window.scrollY) {
      setBgPosition(layer01, -1.2);
      setTransform(layer05R, -0.1, "1");
      setTransform(layer05L, -0.2, "1");
      setBgPosition(layer06, 0.5);
      setTransform(layer10, -0.02, "1");
      // Move mask for header
      if (main) {
        main.style.maskPositionY = window.scrollY - 170 + "px";
        main.style.webkitMaskPositionY = window.scrollY - 170 + "px";

        if (window.scrollY < 10) {
          main.style.webkitMaskImage = "none";
          main.style.maskImage = "none";
        } else {
          main.style.webkitMaskImage =
            "linear-gradient(180deg, transparent 0em, transparent 80px, rgba(0, 0, 0, 1) 170px)";
          main.style.maskImage = "linear-gradient(180deg, transparent 0em, transparent 80px, rgba(0, 0, 0, 1) 170px)";
        }
      }
    }
    lastPos = window.scrollY;
    window.requestAnimationFrame(step);
  }
  window.requestAnimationFrame(step);
});

var initParticles = (callback) => {
  if (document.readyState != "loading") callback();
  else document.addEventListener("DOMContentLoaded", callback);
};

initParticles(() => {
  /*
   *  Particles.js
   *
   *  Particle emitter and renderer.
   *
   *  TODO: Gleichmäßige Particle verteilung
   *  TODO: Pulse Animation anpassen (easing)
   */

  // The Fast-Memoize Library:
  // https://github.com/caiogondim/fast-memoize.js
  /*
   *  Particles.js
   *
   *  Particle emitter and renderer.
   *
   *  TODO: Gleichmäßige Particle verteilung
   *  TODO: Pulse Animation anpassen (easing)
   */

  // The Fast-Memoize Library:
  // https://github.com/caiogondim/fast-memoize.js

  var Memoize = (function () {
    //
    // Main
    //

    function memoize(fn, options) {
      var cache = options && options.cache ? options.cache : cacheDefault;

      var serializer = options && options.serializer ? options.serializer : serializerDefault;

      var strategy = options && options.strategy ? options.strategy : strategyDefault;

      return strategy(fn, {
        cache: cache,
        serializer: serializer,
      });
    }

    //
    // Strategy
    //

    function isPrimitive(value) {
      return value == null || typeof value === "number" || typeof value === "boolean"; // || typeof value === "string" 'unsafe' primitive for our needs
    }

    function monadic(fn, cache, serializer, arg) {
      var cacheKey = isPrimitive(arg) ? arg : serializer(arg);

      var computedValue = cache.get(cacheKey);
      if (typeof computedValue === "undefined") {
        computedValue = fn.call(this, arg);
        cache.set(cacheKey, computedValue);
      }

      return computedValue;
    }

    function variadic(fn, cache, serializer) {
      var args = Array.prototype.slice.call(arguments, 3);
      var cacheKey = serializer(args);

      var computedValue = cache.get(cacheKey);
      if (typeof computedValue === "undefined") {
        computedValue = fn.apply(this, args);
        cache.set(cacheKey, computedValue);
      }

      return computedValue;
    }

    function assemble(fn, context, strategy, cache, serialize) {
      return strategy.bind(context, fn, cache, serialize);
    }

    function strategyDefault(fn, options) {
      var strategy = fn.length === 1 ? monadic : variadic;

      return assemble(fn, this, strategy, options.cache.create(), options.serializer);
    }

    function strategyVariadic(fn, options) {
      var strategy = variadic;

      return assemble(fn, this, strategy, options.cache.create(), options.serializer);
    }

    function strategyMonadic(fn, options) {
      var strategy = monadic;

      return assemble(fn, this, strategy, options.cache.create(), options.serializer);
    }

    //
    // Serializer
    //

    function serializerDefault() {
      return JSON.stringify(arguments);
    }

    //
    // Cache
    //

    function ObjectWithoutPrototypeCache() {
      this.cache = Object.create(null);
    }

    ObjectWithoutPrototypeCache.prototype.has = function (key) {
      return key in this.cache;
    };

    ObjectWithoutPrototypeCache.prototype.get = function (key) {
      return this.cache[key];
    };

    ObjectWithoutPrototypeCache.prototype.set = function (key, value) {
      this.cache[key] = value;
    };

    var cacheDefault = {
      create: function create() {
        return new ObjectWithoutPrototypeCache();
      },
    };

    //
    // API
    //

    // Default export
    var exports = memoize;
    exports.strategies = {
      variadic: strategyVariadic,
      monadic: strategyMonadic,
    };

    return exports;
  })();

  // Create the Particles Module | Namespace.
  var Particles = (function () {
    //
    //  Helper
    //
    var MathRandom = Math.random;
    var MathAbs = Math.abs;
    var MathTrunc = Math.trunc;
    var MathPow = Math.pow;

    function randomValue(min, max) {
      var value = MathRandom() * (+max - +min) + +min;
      // Prevent values that can be possible boring
      return min !== 0 && max !== 0 && (value === 0 || value === 1) ? randomValue(min, max) : value;
    }

    function getNodeYOffset(node) {
      var dimensions = node.getBoundingClientRect();
      return dimensions ? dimensions.y : 0;
    }

    function getNodeHeight(node) {
      var dimensions = node.getBoundingClientRect();
      return dimensions ? dimensions.height : window.innerHeight;
    }

    function calcParticleCount(scene, particleDensity) {
      var referenceArea = 100 * 100; // 100px * 100px => px^2
      var sceneVolume = scene.width * scene.height;
      var scale = 100;
      var particleFactor = sceneVolume / referenceArea / scale;
      var particleCount = MathAbs(MathTrunc(particleFactor * particleDensity));
      return particleCount || 0;
    }

    var normalize = Memoize(function doNormalize(val) {
      var min = 0;
      var max = 1;
      var delta = max - min;
      return (val - min) / delta;
    });

    // Easing functions from: https://easings.net/

    var easeOutQuint = Memoize(function doEaseOutQuint(x) {
      return 1 - MathPow(1 - x, 5);
    });

    var easeOutExpo = Memoize(function easeOutExpo(x) {
      return x >= 1 ? 1 : 1 - MathPow(2, -10 * x);
    });

    function easing(value) {
      return value < 0 ? -easeOutExpo(-value) : easeOutExpo(value);
    }

    //
    //  Data
    //

    function Point2D(x, y) {
      return { x, y };
    }

    function MinMax(min, max) {
      return { min, max };
    }

    function RandomPoint2D(minMaxX, minMaxY) {
      var x = randomValue(minMaxX.min, minMaxX.max);
      var y = randomValue(minMaxY.min, minMaxY.max);
      return Point2D(x, y);
    }

    //
    //  Transformations | Actions
    //

    function Update(scene, particle) {
      // Cache object access -> Performance
      var width = scene.width;
      var height = scene.height;
      var color = particle.color;
      var particleSize = particle.size;
      var particleSizeStep = particleSize.step;
      var particleSizeAnimValue = particleSize.animValue;
      var particleSizeScale = particleSize.scale;
      var particleSizeDirection = particleSize.direction;
      var particlePosition = particle.position;
      var particlePositionX = particlePosition.x;
      var particlePositionY = particlePosition.y;
      var particleDirection = particle.direction;
      var particleDirectionX = particleDirection.x;
      var particleDirectionY = particleDirection.y;

      // Animate the pulse effect of the particle
      //
      // var absSizeAnimValue = particleSizeAnimValue * particleSizeDirection
      var isMaxSize = particleSizeAnimValue > 1;
      var isMinSize = particleSizeAnimValue < -1;
      var sizeDirection = isMaxSize || isMinSize ? -particleSizeDirection : particleSizeDirection;
      var animValue = particleSizeAnimValue + particleSizeStep * sizeDirection;
      // The operator ~~ is a more performant shorthand for Math.trunc()
      var sizeValue = ~~(easing(animValue) * particleSizeScale);

      // Animate the movement of the particle
      //
      var sizeOffset = sizeValue * 0.5;
      var top = particlePositionY + sizeOffset;
      var bottom = particlePositionY - sizeOffset;
      var left = particlePositionX - sizeOffset;
      var right = particlePositionX + sizeOffset;

      var isRightEdge = left > width + 1;
      var isLeftEdge = right < -1;
      var isTopEdge = bottom < -1;
      var isBottomEdge = top > height + 1;

      var positionX = isRightEdge
        ? -sizeOffset
        : isLeftEdge
          ? width + sizeOffset
          : particlePositionX + particleDirectionX;
      var positionY = isBottomEdge
        ? -sizeOffset
        : isTopEdge
          ? height + sizeOffset
          : particlePositionY + particleDirectionY;
      return {
        position: {
          x: positionX,
          y: positionY,
        },
        direction: particleDirection,
        size: {
          value: sizeValue,
          animValue: animValue,
          step: particleSizeStep,
          scale: particleSizeScale,
          direction: sizeDirection,
          offset: sizeOffset,
        },
        color: color,
      };
    }

    function Draw(scene, particle) {
      // Cache object access
      var particlePosition = particle.position;
      var particlePositionX = particlePosition.x;
      var particlePositionY = particlePosition.y;
      var particleSize = particle.size;
      var particleSizeValue = particleSize.value;
      var drawOffset = particleSize.offset;

      // Draw calculations --
      var yOffset = scene.yOffset * scene.depthScale;
      var size = particleSizeValue;
      var x = particlePositionX - drawOffset;
      var y = yOffset + (particlePositionY - drawOffset);

      var ctx = scene.ctx;
      // var isVisible = (size > 0 && y < window.innerHeight && y > 0)
      var isVisible = size > 0;

      // Draw
      if (isVisible) {
        ctx.fillStyle = particle.color;
        // Round to full integer to prevent subpixel rendering
        //  - The ~~ operator is equivalent to Math.trunc() but faster.
        // ctx.fillRect(~~x, ~~y, size, size)
        ctx.fillRect(x, y, size, size);
      }
    }

    function Clear(scene) {
      scene.ctx.clearRect(0, 0, scene.width, scene.height);
    }

    function Render(scene, particles) {
      Clear(scene);
      particles.forEach(function doDraw(particle) {
        Draw(scene, particle);
      });
    }

    function Animate(scene, particles) {
      var updatedParticles = [],
        fpsInterval = scene.fpsInterval,
        now = performance.now(),
        then = scene.then,
        elapsed = now - then,
        tolerance = 0.1;

      // Just render when we're reaching the configured fps
      if (elapsed >= scene.fpsInterval - tolerance) {
        scene.yOffset = getNodeYOffset(scene.anchorNode);
        scene.then = now - (elapsed % fpsInterval);
        Render(scene, particles);
      }

      // Update particle positions for the next render call
      updatedParticles = particles.map(function doUpdate(p) {
        return Update(scene, p);
      });

      // Function to animate the next frame
      return function nextFrame() {
        return Animate(scene, updatedParticles);
      };
    }

    function CreateScene(cfg) {
      var canvasId = cfg.canvasId,
        contentId = cfg.contentId,
        depth = cfg.depth,
        fps = cfg.fps;

      var contentNode = document.getElementById(contentId),
        width = window.innerWidth,
        fpsInterval = 1000 / fps,
        yOffset = 0,
        canvas = document.getElementById(canvasId),
        height = getNodeHeight(canvas),
        ctx = canvas.getContext("2d"),
        depthScaledHeight = height * (depth + 1);

      ctx.canvas.width = width;
      ctx.canvas.height = window.innerHeight;

      return {
        width: width,
        height: depthScaledHeight,
        yOffset: yOffset,
        depthScale: depth,
        fps: fps,
        fpsInterval: fpsInterval,
        then: performance.now(),
        anchorNode: contentNode,
        ctx: ctx,
      };
    }

    function CreateParticles(scene, cfg) {
      var density = cfg.density,
        color = cfg.color,
        size = cfg.size,
        speed = cfg.speed,
        lifespan = cfg.lifespan;

      var initAnimValue = randomValue(-1, 1);

      return Array.from(
        {
          // length: 1
          length: calcParticleCount(scene, density),
        },
        function Particle() {
          const lifeLength = randomValue(lifespan.min, lifespan.max);
          return {
            position: RandomPoint2D(MinMax(0, scene.width - size * 2), MinMax(0, scene.height - size * 2)),
            direction: RandomPoint2D(MinMax(-speed, speed), MinMax(-speed, speed)),
            size: {
              animValue: initAnimValue,
              scale: size,
              value: size * initAnimValue,
              step: 1 / (lifeLength / scene.fps),
              direction:
                // 50% Chance for the particle floating in one
                // direction or the other.
                randomValue(0, 100) < 50 ? -1 : +1,
              // Offset is needed for boundary calculation and for
              // drawing the pixels. We cache it here, to prevent
              // double calculation of the same value.
              offset: size * initAnimValue * 0.5,
            },
            color,
          };
        },
      );
    }

    //
    //  Api
    //

    function Particles(configs) {
      var particles = [];
      var isRunning = false;
      var throttled = false;
      var delay = 250; // ms
      var rafId = undefined;

      function animate(rendererList) {
        if (isRunning) {
          rafId = requestAnimationFrame(function doRaf() {
            animate(
              rendererList.map(function doRender(render) {
                return render();
              }),
            );
          });
        }
      }

      function init() {
        particles = configs.map(function (particleConfig) {
          var scene = CreateScene(particleConfig.scene);
          var particles = CreateParticles(scene, particleConfig.particles);
          return Animate(scene, particles);
        });
        return particles;
      }

      function run() {
        if (particles.length === 0) {
          particles = init();
        }
        isRunning = true;
        animate(particles);
      }

      function stop() {
        isRunning = false;
        cancelAnimationFrame(rafId);
      }

      return {
        init: init,
        run: run,
        stop: stop,
        onScreenSizeChange: function onResize() {
          stop();
          if (!throttled) {
            throttled = true;
            setTimeout(function () {
              particles = init();
              run();
              throttled = false;
            }, delay);
          }
        },
      };
    }

    // Default Export
    return Particles;
  })();

  var fgParticleConfig = {
    scene: {
      // Id of the canvas to render to
      canvasId: "fg-particles",
      // Id of the content container, we're using it to get the
      // rendering height and it's the anchor to get the y offset
      // to simulate scrolling.
      contentId: "content",
      fps: 24,
      depth: 2.5,
    },
    particles: {
      density: 8,
      color: "white",
      size: 20,
      speed: 0.7,
      lifespan: {
        // in milliseconds
        min: 3500,
        max: 7000,
      },
    },
  };
  var bgParticleConfig = {
    scene: {
      // Id of the canvas to render to
      canvasId: "bg-particles",
      // Id of the content container, we're using it to get the
      // rendering height and it's the anchor to get the y offset
      // to simulate scrolling.
      contentId: "content",
      fps: 16,
      depth: 0.08,
    },
    particles: {
      density: 110,
      color: "white",
      size: 6,
      speed: 0.4,
      lifespan: {
        // in milliseconds
        min: 2500,
        max: 5000,
      },
    },
  };
  var animation = Particles([fgParticleConfig, bgParticleConfig]);
  animation.run();
  window.addEventListener("resize", animation.onScreenSizeChange);
});
